package CF2022::Controller::Orders;
use Mojo::Base 'Mojolicious::Controller';
use Mojo::UserAgent;

use constant MAIN_PRODUCTS => qr/^(1|2|3)$/;

use feature 'signatures';
no warnings qw{ experimental::signatures };

sub create ($c ) {
    $c->openapi->valid_input or return;

    my $args      = $c->req->json;
    my @products  = @{ $args->{products} };
    my @responses = @{ $args->{responses} };
    my $claims;
    my $group;

    my $url = sprintf ('%s/organizers/%s/events/%s/orders/',
        $c->config->{pretix_api},
        $c->config->{pretix_organizer},
        $c->config->{pretix_event},
    );

    if ( $args->{token} ) {
        $c->oauth_token($args->{token});

        $claims = $c->oauth_claims;
        return $c->error(403, 'Invalid token') if ! $claims;

        $group = $c->oauth_main_group;
        return $c->error(403, 'Invalid token') if ! $group;

        $args->{name} = $claims->{name};

        push @responses, (
            {
                question_id => $c->config->{pretix_qid_sso},
                response    => $claims->{sub},
            },
            {
                question_id => $c->config->{pretix_qid_region},
                response    => $group->{region},
            },
        );

        @products = grep { $_->{id} !~ MAIN_PRODUCTS } @products;
        push @products, {
            id => $c->config->{'pretix_product_' . $group->{role}}
        };

        if ( $group->{role} eq 'member' ) {
            $args->{email} = $claims->{preferred_username}
                           .'@'. $c->config->{mail_domain};
        }
        else {
            $args->{email} = $claims->{email};
        }
    }

    my $order = {
        email            => $args->{email},
        locale           => 'en',
        payment_provider => 'manual',
        testmode         => $c->config->{pretix_testmode} ? 'true' : 'false',
        positions        => [],
        fees             => [],
    };

    PRODUCT:
    foreach my $product ( @products ) {
        my $item = {
            item      => $product->{id},
            variation => $product->{variation},
            attendee_name => $args->{name},
            price => $product->{price},
            company => $args->{company},
        };

        if ( $product->{id} =~ MAIN_PRODUCTS ) {
            RESPONSE:
            foreach my $response ( @responses ) {
                push @{ $item->{answers} }, {
                    question => $response->{question_id},
                    answer   => $response->{response},
                };
            }
        }

        push @{ $order->{positions} }, $item;
    }

    # kontrola duplicity

    my $exists = $c->schema->resultset('Order')->search({
        email => $order->{email},
        api   => $url,
    })->count;

    $exists ||= $c->schema->resultset('Order')->search({
        sso_uuid => $claims->{sub},
        api      => $url,
    })->count if $claims;

    return $c->error(400, 'Duplicity' ) if $exists;

    # odeslani do pretixu

    my $ua = Mojo::UserAgent->new;
    my $rc = $ua->post( $url,
        { Authorization => 'Token ' . $c->config->{pretix_token} },
        json => $order
    )->result;

    return $c->error(400, $rc->json) if ! $rc->is_success;

    # ukladani do lokalni databazi

    $c->schema->resultset('Order')->create({
        ip       => $c->tx->remote_address,
        sso_uuid => $claims->{sub},
        email    => $order->{email},
        api      => $url,
        request  => $order,
        response => $rc->json,
    });

    $c->render(
        status => 201,
        json   => $rc->json,
    );
}

1;
